package waluty;

import java.io.InputStream;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.net.http.HttpResponse.BodyHandlers;
import java.time.LocalDate;

import jakarta.json.Json;
import jakarta.json.JsonArray;
import jakarta.json.JsonObject;
import jakarta.json.JsonReader;
import jakarta.json.JsonValue;

public class PobieranieJSON extends Pobieranie {
    private HttpClient httpClient = HttpClient.newHttpClient();
    
    @Override
    public Tabela pobierzBiezaceKursy() throws BladAplikacji {
        return pobierzZAdresu(ADRES_BAZOWY + "/A" + "?format=json");
    }
    
    @Override
    public Tabela pobierzArchiwalneKursy(String data) throws BladAplikacji {
        return pobierzZAdresu(ADRES_BAZOWY + "/A/" + data + "?format=json");
    }

    private Tabela pobierzZAdresu(String adres) throws BladAplikacji {
        try {
            URI uri = new URI(adres);
            HttpRequest request = HttpRequest.newBuilder(uri).build();
            HttpResponse<InputStream> response = httpClient.send(request, BodyHandlers.ofInputStream());
            if(response.statusCode() != 200) {
                throw new BladAplikacji("Brak danych w odpowiedzi HTTP, kod " + response.statusCode());
            }

            return tabelaZJSON(response.body());
        } catch(BladAplikacji e) {
            throw e;
        } catch(Exception e) {
            throw new BladAplikacji("Błąd podczas pobierania XML: " + e.getMessage(), e);
        }
    }

    private Tabela tabelaZJSON(InputStream input) {
        try(JsonReader reader = Json.createReader(input)) {
            // dokument zawiera [tablicę], a ta tablica zawiera {obiekt}
            JsonArray array = reader.readArray();
            JsonObject tabela = array.getJsonObject(0);
            // String typ = tabela.getString("table");
            String no = tabela.getString("no");
            LocalDate date = LocalDate.parse(tabela.getString("effectiveDate"));

            Tabela wynikowaTabela = new Tabela(no, date);

            JsonArray rates = tabela.getJsonArray("rates");
            for(JsonValue jsonValue : rates) {
                JsonObject jsonObject = jsonValue.asJsonObject();
                Waluta waluta = new Waluta(jsonObject.getString("code"),
                        jsonObject.getString("currency"),
                        jsonObject.getJsonNumber("mid").bigDecimalValue());
                wynikowaTabela.addRate(waluta);
            }

            return wynikowaTabela;
        }
    }
}
